@@ -211,7 +211,8 @@ gst_atomic_int_dec_and_test (GstAtomicInt *aint)
 }
 
 /***** Sun SPARC *****/
-#elif defined(HAVE_CPU_SPARC) && defined(__GNUC__)
+#elif 0 && defined(HAVE_CPU_SPARC) && defined(__GNUC__)
+/* allegedly broken again */
 
 GST_INLINE_FUNC void 	gst_atomic_int_destroy 	(GstAtomicInt *aint) { } 
 
@@ -382,6 +383,7 @@ gst_atomic_int_dec_and_test (GstAtomicInt *aint)
 
 /***** S/390 *****/
 #elif defined(HAVE_CPU_S390) && defined(__GNUC__)
+typedef struct { volatile int counter; } atomic_t __attribute__ ((aligned (4)));
 
 GST_INLINE_FUNC void 	gst_atomic_int_init 	(GstAtomicInt *aint, gint val) { aint->counter = val; }
 GST_INLINE_FUNC void 	gst_atomic_int_destroy 	(GstAtomicInt *aint) { } 
@@ -421,7 +423,11 @@ gst_atomic_int_dec_and_test (GstAtomicInt *aint)
 }
 
 #else 
+
+/* no need warning about this if we can't do inline assembly */
+#ifdef __GNUC__
 #warning consider putting your architecture specific atomic implementations here
+#endif
 
 /*
  * generic implementation
