@@ -22,7 +22,6 @@
  */
 
 #include "gst_private.h"
-
 #include "gstinfo.h"
 
 #ifndef GST_DISABLE_GST_DEBUG
@@ -34,17 +33,20 @@
 #include <printf.h>
 #endif
 #include <stdio.h>              /* fprintf */
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <string.h>             /* G_VA_COPY */
-#include "gstinfo.h"
-#include "gstlog.h"
-#include "gst_private.h"
 #include "gstelement.h"
 #include "gstpad.h"
 #include "gstscheduler.h"
 #include "gst_private.h"
+#ifdef HAVE_VALGRIND
+#include <valgrind/valgrind.h>
+#endif
 
-GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEBUG);
+/* underscore is to prevent conflict with GST_CAT_DEBUG define */
+GST_DEBUG_CATEGORY_STATIC (_GST_CAT_DEBUG);
 
 #if 0
 #if defined __sgi__
@@ -58,7 +60,8 @@ typedef struct DL_INFO
   int dli_version;
   int dli_reserved1;
   long dli_reserved[4];
-} Dl_info;
+}
+Dl_info;
 
 #define _RLD_DLADDR             14
 int dladdr (void *address, Dl_info * dl);
@@ -74,8 +77,6 @@ dladdr (void *address, Dl_info * dl)
 #endif /* __sgi__ */
 #endif
 
-extern gchar *_gst_progname;
-
 static void gst_debug_reset_threshold (gpointer category, gpointer unused);
 static void gst_debug_reset_all_thresholds (void);
 
@@ -100,7 +101,8 @@ typedef struct
 {
   GPatternSpec *pat;
   GstDebugLevel level;
-} LevelNameEntry;
+}
+LevelNameEntry;
 
 /* list of all categories */
 static GStaticMutex __cat_mutex = G_STATIC_MUTEX_INIT;
@@ -111,7 +113,8 @@ typedef struct
 {
   GstLogFunction func;
   gpointer user_data;
-} LogFuncEntry;
+}
+LogFuncEntry;
 static GStaticMutex __log_func_mutex = G_STATIC_MUTEX_INIT;
 static GSList *__log_functions = NULL;
 
@@ -131,7 +134,6 @@ GstDebugCategory *GST_CAT_PARENTAGE = NULL;
 GstDebugCategory *GST_CAT_STATES = NULL;
 GstDebugCategory *GST_CAT_PLANNING = NULL;
 GstDebugCategory *GST_CAT_SCHEDULING = NULL;
-GstDebugCategory *GST_CAT_DATAFLOW = NULL;
 GstDebugCategory *GST_CAT_BUFFER = NULL;
 GstDebugCategory *GST_CAT_CAPS = NULL;
 GstDebugCategory *GST_CAT_CLOCK = NULL;
@@ -152,6 +154,41 @@ GstDebugCategory *GST_CAT_PARAMS = NULL;
 GstDebugCategory *GST_CAT_CALL_TRACE = NULL;
 GstDebugCategory *GST_CAT_SEEK = NULL;
 
+/* FIXME: export this? */
+gboolean
+__gst_in_valgrind (void)
+{
+  static enum
+  {
+    GST_VG_UNCHECKED,
+    GST_VG_NO_VALGRIND,
+    GST_VG_INSIDE
+  }
+  in_valgrind = GST_VG_UNCHECKED;
+
+  if (in_valgrind == GST_VG_UNCHECKED) {
+#ifdef HAVE_VALGRIND
+    if (RUNNING_ON_VALGRIND) {
+      GST_CAT_INFO (GST_CAT_GST_INIT, "we're running inside valgrind");
+      VALGRIND_PRINTF
+          ("GStreamer has detected that it is running inside valgrind.");
+      VALGRIND_PRINTF
+          ("It might now take different code paths to ease debugging.");
+      VALGRIND_PRINTF ("Of course, this may also lead to different bugs.");
+      in_valgrind = GST_VG_INSIDE;
+    } else {
+      GST_CAT_LOG (GST_CAT_GST_INIT, "not doing extra valgrind stuff");
+      in_valgrind = GST_VG_NO_VALGRIND;
+    }
+#else
+    in_valgrind = GST_VG_NO_VALGRIND;
+#endif
+    g_assert (in_valgrind == GST_VG_NO_VALGRIND ||
+        in_valgrind == GST_VG_INSIDE);
+  }
+  return (in_valgrind == GST_VG_INSIDE) ? TRUE : FALSE;
+}
+
 /**
  * _gst_debug_init:
  * 
@@ -172,7 +209,7 @@ _gst_debug_init (void)
   /* do NOT use a single debug function before this line has been run */
   GST_CAT_DEFAULT = _gst_debug_category_new ("default",
       GST_DEBUG_UNDERLINE, NULL);
-  GST_CAT_DEBUG = _gst_debug_category_new ("GST_DEBUG",
+  _GST_CAT_DEBUG = _gst_debug_category_new ("GST_DEBUG",
       GST_DEBUG_BOLD | GST_DEBUG_FG_YELLOW, "debugging subsystem");
 
   gst_debug_add_log_function (gst_debug_log_default, NULL);
@@ -196,8 +233,6 @@ _gst_debug_init (void)
       GST_DEBUG_BOLD | GST_DEBUG_FG_MAGENTA, NULL);
   GST_CAT_SCHEDULING = _gst_debug_category_new ("GST_SCHEDULING",
       GST_DEBUG_BOLD | GST_DEBUG_FG_MAGENTA, NULL);
-  GST_CAT_DATAFLOW = _gst_debug_category_new ("GST_DATAFLOW",
-      GST_DEBUG_BOLD | GST_DEBUG_FG_GREEN, NULL);
   GST_CAT_BUFFER = _gst_debug_category_new ("GST_BUFFER",
       GST_DEBUG_BOLD | GST_DEBUG_FG_GREEN, NULL);
   GST_CAT_CAPS = _gst_debug_category_new ("GST_CAPS",
@@ -237,10 +272,13 @@ _gst_debug_init (void)
       GST_DEBUG_BOLD, NULL);
   GST_CAT_SEEK = _gst_debug_category_new ("GST_SEEK",
       0, "plugins reacting to seek events");
+
+  /* print out the valgrind message if we're in valgrind */
+  __gst_in_valgrind ();
 }
 
 /* we can't do this further above, because we initialize the GST_CAT_DEFAULT struct */
-#define GST_CAT_DEFAULT GST_CAT_DEBUG
+#define GST_CAT_DEFAULT _GST_CAT_DEBUG
 
 /**
  * gst_debug_log:
@@ -314,8 +352,10 @@ gst_debug_log_valist (GstDebugCategory * category, GstDebugLevel level,
  * gst_debug_message_get:
  * @message: a debug message
  *
- * Gets the string representation of a GstDebugMessage. This function is used
+ * Gets the string representation of a #GstDebugMessage. This function is used
  * in debug handlers to extract the message.
+ *
+ * Returns: the string representation of a #GstDebugMessage.
  */
 const gchar *
 gst_debug_message_get (GstDebugMessage * message)
@@ -439,6 +479,7 @@ gst_debug_log_default (GstDebugCategory * category, GstDebugLevel level,
   gchar *obj;
   gchar *pidcolor;
   gint pid;
+  GTimeVal now;
 
   if (level > gst_debug_category_get_threshold (category))
     return;
@@ -460,8 +501,10 @@ gst_debug_log_default (GstDebugCategory * category, GstDebugLevel level,
 
   obj = object ? gst_debug_print_object (object) : g_strdup ("");
 
-  g_printerr ("%s %s%15s%s(%s%5d%s) %s%s(%d):%s:%s%s %s\n",
+  g_get_current_time (&now);
+  g_printerr ("%s (%10ld:%06ld) %s%15s%s(%s%5d%s) %s%s(%d):%s:%s%s %s\n",
       gst_debug_level_get_name (level),
+      now.tv_sec, now.tv_usec,
       color, gst_debug_category_get_name (category), clear,
       pidcolor, pid, clear,
       color, file, line, function, obj, clear, gst_debug_message_get (message));
