@@ -25,6 +25,7 @@
 
 #include <string.h>
 
+#include "gst_private.h"
 #include <gst/gst.h>
 #include <gobject/gvaluecollector.h>
 
@@ -224,6 +225,7 @@ gst_structure_free (GstStructure * structure)
       g_value_unset (&field->value);
     }
   }
+  g_array_free (structure->fields, TRUE);
 #ifdef USE_POISONING
   memset (structure, 0xff, sizeof (GstStructure));
 #endif
@@ -246,6 +248,22 @@ gst_structure_get_name (const GstStructure * structure)
   return g_quark_to_string (structure->name);
 }
 
+/**
+ * gst_structure_get_name:
+ * @structure: a #GstStructure
+ *
+ * Accessor fuction.
+ *
+ * Returns: the quark representing the name of the structure.
+ */
+GQuark
+gst_structure_get_name_id (const GstStructure * structure)
+{
+  g_return_val_if_fail (structure != NULL, 0);
+
+  return structure->name;
+}
+
 /**
  * gst_structure_set_name:
  * @structure: a #GstStructure
@@ -405,6 +423,11 @@ gst_structure_set_valist (GstStructure * structure,
 
           g_value_init (&field.value, GST_TYPE_DOUBLE_RANGE);
           gst_value_set_double_range (&field.value, min, max);
+        } else if (type == GST_TYPE_BUFFER) {
+          GstBuffer *buffer = va_arg (varargs, GstBuffer *);
+
+          g_value_init (&field.value, GST_TYPE_BUFFER);
+          g_value_set_boxed (&field.value, buffer);
         } else {
           g_critical ("unimplemented vararg field type %d\n", (int) type);
           return;
@@ -951,7 +974,8 @@ typedef struct _GstStructureAbbreviation
 {
   char *type_name;
   GType type;
-} GstStructureAbbreviation;
+}
+GstStructureAbbreviation;
 
 static GstStructureAbbreviation gst_structure_abbrs[] = {
   {"int", G_TYPE_INT},
@@ -986,6 +1010,9 @@ gst_structure_from_abbr (const char *type_name)
   if (strcmp (type_name, "fourcc") == 0 || strcmp (type_name, "4") == 0) {
     return GST_TYPE_FOURCC;
   }
+  if (strcmp (type_name, "buffer") == 0) {
+    return GST_TYPE_BUFFER;
+  }
 
   return g_type_from_name (type_name);
 }
@@ -1007,10 +1034,35 @@ gst_structure_to_abbr (GType type)
   if (type == GST_TYPE_FOURCC) {
     return "fourcc";
   }
+  if (type == GST_TYPE_BUFFER) {
+    return "buffer";
+  }
 
   return g_type_name (type);
 }
 
+static GType
+gst_structure_value_get_generic_type (GValue * val)
+{
+  if (G_VALUE_TYPE (val) == GST_TYPE_LIST
+      || G_VALUE_TYPE (val) == GST_TYPE_FIXED_LIST) {
+    GArray *array = g_value_peek_pointer (val);
+
+    if (array->len > 0) {
+      GValue *value = &g_array_index (array, GValue, 0);
+
+      return gst_structure_value_get_generic_type (value);
+    } else {
+      return G_TYPE_INT;
+    }
+  } else if (G_VALUE_TYPE (val) == GST_TYPE_INT_RANGE) {
+    return G_TYPE_INT;
+  } else if (G_VALUE_TYPE (val) == GST_TYPE_DOUBLE_RANGE) {
+    return G_TYPE_DOUBLE;
+  }
+  return G_VALUE_TYPE (val);
+}
+
 #define GST_ASCII_IS_STRING(c) (g_ascii_isalnum((c)) || ((c) == '_') || \
       ((c) == '-') || ((c) == '+') || ((c) == '/') || ((c) == ':') || \
       ((c) == '.'))
@@ -1048,23 +1100,8 @@ gst_structure_to_string (const GstStructure * structure)
     field = GST_STRUCTURE_FIELD (structure, i);
 
     t = gst_value_serialize (&field->value);
-    type = G_VALUE_TYPE (&field->value);
+    type = gst_structure_value_get_generic_type (&field->value);
 
-    if (type == GST_TYPE_LIST) {
-      GArray *array = g_value_peek_pointer (&field->value);
-
-      if (array->len > 0) {
-        GValue *value = &g_array_index (array, GValue, 0);
-
-        type = G_VALUE_TYPE (value);
-      } else {
-        type = G_TYPE_INT;
-      }
-    } else if (G_VALUE_TYPE (&field->value) == GST_TYPE_INT_RANGE) {
-      type = G_TYPE_INT;
-    } else if (G_VALUE_TYPE (&field->value) == GST_TYPE_DOUBLE_RANGE) {
-      type = G_TYPE_DOUBLE;
-    }
     g_string_append_printf (s, ", %s=(%s)%s", g_quark_to_string (field->name),
         gst_structure_to_abbr (type), t);
     g_free (t);
@@ -1182,22 +1219,23 @@ gst_structure_parse_range (gchar * s, gchar ** after, GValue * value,
 }
 
 static gboolean
-gst_structure_parse_list (gchar * s, gchar ** after, GValue * value, GType type)
+gst_structure_parse_any_list (gchar * s, gchar ** after, GValue * value,
+    GType type, GType list_type, char begin, char end)
 {
   GValue list_value = { 0 };
   gboolean ret;
   GArray *array;
 
-  g_value_init (value, GST_TYPE_LIST);
+  g_value_init (value, list_type);
   array = g_value_peek_pointer (value);
 
-  if (*s != '{')
+  if (*s != begin)
     return FALSE;
   s++;
 
   while (g_ascii_isspace (*s))
     s++;
-  if (*s == '}') {
+  if (*s == end) {
     s++;
     *after = s;
     return TRUE;
@@ -1212,7 +1250,7 @@ gst_structure_parse_list (gchar * s, gchar ** after, GValue * value, GType type)
   while (g_ascii_isspace (*s))
     s++;
 
-  while (*s != '}') {
+  while (*s != end) {
     if (*s != ',')
       return FALSE;
     s++;
@@ -1236,6 +1274,21 @@ gst_structure_parse_list (gchar * s, gchar ** after, GValue * value, GType type)
   return TRUE;
 }
 
+static gboolean
+gst_structure_parse_list (gchar * s, gchar ** after, GValue * value, GType type)
+{
+  return gst_structure_parse_any_list (s, after, value, type, GST_TYPE_LIST,
+      '{', '}');
+}
+
+static gboolean
+gst_structure_parse_fixed_list (gchar * s, gchar ** after, GValue * value,
+    GType type)
+{
+  return gst_structure_parse_any_list (s, after, value, type,
+      GST_TYPE_FIXED_LIST, '<', '>');
+}
+
 static gboolean
 gst_structure_parse_simple_string (gchar * str, gchar ** end)
 {
@@ -1339,6 +1392,8 @@ gst_structure_parse_value (gchar * str,
     ret = gst_structure_parse_range (s, &s, value, type);
   } else if (*s == '{') {
     ret = gst_structure_parse_list (s, &s, value, type);
+  } else if (*s == '<') {
+    ret = gst_structure_parse_fixed_list (s, &s, value, type);
   } else {
     value_s = s;
     if (!gst_structure_parse_string (s, &value_end, &s))
