@@ -29,9 +29,27 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <dirent.h>
-#include <unistd.h>
 #include <fcntl.h>
+#ifdef _MSC_VER
+#include <sys/utime.h>
+#include <io.h>
+#ifndef F_OK
+#define F_OK 0
+#define W_OK 2
+#define R_OK 4
+#endif
+#ifndef S_ISREG
+#define S_ISREG(mode) ((mode)&_S_IFREG)
+#endif
+#ifndef S_ISDIR
+#define S_ISDIR(mode) ((mode)&_S_IFDIR)
+#endif
+#else /* _MSC_VER */
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include <utime.h>
+#endif
 
 #include <gst/gst_private.h>
 #include <gst/gstelement.h>
@@ -300,9 +318,11 @@ get_time (const char *path, gboolean * is_dir)
   return statbuf.st_ctime;
 }
 
-/* same as 0755 */
-#define DIRMODE \
-  (S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)
+#ifdef _MSC_VER
+#define xmkdir(dirname) _mkdir (dirname)
+#else
+#define xmkdir(dirname) mkdir (dirname, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)
+#endif
 
 static gboolean
 make_dir (gchar * filename)
@@ -316,19 +336,16 @@ make_dir (gchar * filename)
   dirname = g_strndup (filename, strrchr (filename, '/') - filename);
 
   if (stat (dirname, &dirstat) == -1 && errno == ENOENT) {
-#ifndef HAVE_WIN32
-    if (mkdir (dirname, DIRMODE) != 0) {
+    if (xmkdir (dirname) != 0) {
       if (make_dir (dirname) != TRUE) {
         g_free (dirname);
         return FALSE;
       } else {
-        if (mkdir (dirname, DIRMODE) != 0)
+        if (xmkdir (dirname) != 0) {
           return FALSE;
+        }
       }
     }
-#else
-    return FALSE;
-#endif
   }
 
   g_free (dirname);
@@ -757,9 +774,9 @@ gst_xml_registry_parse_element_factory (GMarkupParseContext * context,
       gst_plugin_feature_set_rank (GST_PLUGIN_FEATURE (factory), rank);
     }
   } else if (!strcmp (tag, "uri_type")) {
-    if (strncasecmp (text, "sink", 4) == 0) {
+    if (g_ascii_strncasecmp (text, "sink", 4) == 0) {
       factory->uri_type = GST_URI_SINK;
-    } else if (strncasecmp (text, "source", 5) == 0) {
+    } else if (g_ascii_strncasecmp (text, "source", 5) == 0) {
       factory->uri_type = GST_URI_SRC;
     }
   } else if (!strcmp (tag, "uri_protocol")) {
@@ -863,8 +880,8 @@ gst_xml_registry_parse_padtemplate (GMarkupParseContext * context,
     g_assert (registry->caps == NULL);
     registry->caps = gst_caps_from_string (s);
     if (registry->caps == NULL) {
-      g_critical ("Could not parse caps: length %d, content: %*s\n", text_len,
-          text_len, text);
+      g_critical ("Could not parse caps: length %d, content: %*s\n",
+          (int) text_len, (int) text_len, text);
     }
     g_free (s);
     return TRUE;
